from Globals import InitializeClass
from Products.CMFCalendar.CalendarTool import CalendarTool
from DateTime import DateTime
#from zope.i18nmessageid import MessageIDFactory
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo
import calendar


#_ = MessageIDFactory("plone")
#_eb = MessageIDFactory("easyblog")

class EasyBlogTool(CalendarTool):
    """The EasyBlog Tool"""
    meta_type = "EasyBlogTool"
    id = "easyblog_tool"
    security = ClassSecurityInfo()
    
    def __init__(self):
        #CalendarTool.__init__(self)
        self.calendar_types = ['EasyBlogEntry', ]        

    # taken from CMFCalendar just to change some lines
    # to make entries display blog (resp. path) dependent
    security.declarePublic('getEventsForCalendar')
    def getEventsForCalendar(self, blog, month='1', year='2002'):               # here
        """ recreates a sequence of weeks, by days each day is a mapping.
            {'day': #, 'url': None}
        """

        year=int(year)
        month=int(month)
        # daysByWeek is a list of days inside a list of weeks, like so:
        # [[0, 1, 2, 3, 4, 5, 6],
        #  [7, 8, 9, 10, 11, 12, 13],
        #  [14, 15, 16, 17, 18, 19, 20],
        #  [21, 22, 23, 24, 25, 26, 27],
        #  [28, 29, 30, 31, 0, 0, 0]]
        daysByWeek=calendar.monthcalendar(year, month)
        weeks=[]
       
        events=self.catalog_getevents(blog, year, month)                        # here            

        for week in daysByWeek:
            days=[]
            for day in week:
                if events.has_key(day):
                    days.append(events[day])
                else:
                    days.append({'day': day, 'event': 0, 'eventslist':[]})

            weeks.append(days)

        return weeks

    security.declarePublic('catalog_getevents')
    def catalog_getevents(self, blog, year, month):                             # here
        """ given a year and month return a list of days that have events """
        year=int(year)
        month=int(month)
        last_day=calendar.monthrange(year, month)[1]
        first_date=self.getBeginAndEndTimes(1, month, year)[0]
        last_date=self.getBeginAndEndTimes(last_day, month, year)[1]

        query = self.portal_catalog(
                        portal_type=self.getCalendarTypes(),
                        review_state=self.getCalendarStates(),
                        start={'query': last_date, 'range': 'max'},
                        end={'query': first_date, 'range': 'min'},
                        sort_on='start',                      
                        path = '/'.join(blog.getPhysicalPath())                 # and here
        
            )

        # compile a list of the days that have events
        eventDays={}
        for daynumber in range(1, 32): # 1 to 31
            eventDays[daynumber] = {'eventslist':[], 'event':0, 'day':daynumber}
        includedevents = []       
        for result in query:
            if result.getRID() in includedevents:
                break
            else:
                includedevents.append(result.getRID())
            event={}
            # we need to deal with events that end next month
            if  result.end.month() != month:
                # doesn't work for events that last ~12 months
                # fix it if it's a problem, otherwise ignore
                eventEndDay = last_day
                event['end'] = None
            else:
                eventEndDay = result.end.day()
                event['end'] = result.end.Time()
            # and events that started last month
            if result.start.month() != month:  # same as above re: 12 month thing
                eventStartDay = 1
                event['start'] = None
            else:
                eventStartDay = result.start.day()
                event['start'] = result.start.Time()
            event['title'] = result.Title or result.id
            if eventStartDay != eventEndDay:
                allEventDays = range(eventStartDay, eventEndDay+1)
                eventDays[eventStartDay]['eventslist'].append( {'end': None,
                        'start': result.start.Time(), 'title': result.Title} )
                eventDays[eventStartDay]['event'] = 1
                for eventday in allEventDays[1:-1]:
                    eventDays[eventday]['eventslist'].append( {'end': None,
                                       'start': None, 'title': result.Title} )
                    eventDays[eventday]['event'] = 1
                eventDays[eventEndDay]['eventslist'].append( {'end':
                    result.end.Time(), 'start': None, 'title': result.Title} )
                eventDays[eventEndDay]['event'] = 1
            else:
                eventDays[eventStartDay]['eventslist'].append(event)
                eventDays[eventStartDay]['event'] = 1
            # This list is not uniqued and isn't sorted
            # uniquing and sorting only wastes time
            # and in this example we don't need to because
            # later we are going to do an 'if 2 in eventDays'
            # so the order is not important.
            # example:  [23, 28, 29, 30, 31, 23]
        return eventDays
        
    def sendMail(self, blog, subject, title="", name="", email="", homepage="", ipaddress="", msg=""):      
        notifyTo   = ", ".join( blog.getNotifyTo() )
        notifyFrom = blog.getNotifyFrom()    
        
        message  = "from: %s\n" % notifyFrom
        message += "bcc: %s\n" % notifyTo
        message += "subject: %s\n\n" % subject
        
        message += "Title: %s\n"   % title
        message += "Name: %s\n"   % name
        message += "Mail: %s\n"   % email
        message += "Homepage: %s\n"   % homepage
        message += "IP: %s\n" % ipaddress
        message += "Message: %s\n"   % msg
        
        try:
            self.MailHost.send( message )
        except:
            # catch and do nothing, so that the user doesn't notice an error
            pass

    def getUltimoFor(self, year, month):
        """Returns the ultimo for a given month"""
        
        month = int(month)
        year = int(year)
        
        if month == 12:
            month = 1
            year += 1
        else:
            month += 1
        
        ultimo = DateTime("%s/%s/01" % (year, month))
        ultimo -= 1
                
        return ultimo.dd()

    def getNameOfMonth(self, month):
        """Returns the name of a month given by a int"""
        month = int(month)       
        date = DateTime("2000/%s/01" % month)
        
        return date.Month()

    def getAmountOfEntries(self, start = DateTime("1900/01/01"), end = DateTime("2100/01/01"), category = "", path="/"):
        """Returns the amount of entries by the given parameters"""

        catalog = getToolByName(self, "portal_catalog")
        results = catalog.searchResults(getEntryCategories=category, path=path, portal_type="EasyBlogEntry", review_state="published", Date = {'query': [start, end], 'range': 'minmax' })
        
        return len(results)
        
    def getFullnameById(self, id):
        """Returns the fullname of a Member dependend on the passed id"""
        
        mt = getToolByName(self, "portal_membership")
        fullname = mt.getMemberById(id).fullname
        
        if len(fullname) > 0:
            return fullname     
        return id
        
InitializeClass(EasyBlogTool)
