from Products.Archetypes import listTypes
from Products.Archetypes.Extensions.utils import installTypes, install_subskin
from Products.EasyBlog.workflows import EasyBlogWorkflow, EasyBlogEntryWorkflow, EasyBlogCommentWorkflow, EasyBlogTrackbackWorkflow
from Products.CMFDynamicViewFTI.migrate import migrateFTIs
from Products.CMFCore.utils import getToolByName
from Products.EasyBlog.config import *
from StringIO import StringIO

def install(self):
    out = StringIO()
    
    migrateFTIs(self, product=PROJECTNAME)
        
    propsTool = getToolByName(self, 'portal_properties')
    siteProperties = getattr(propsTool, 'site_properties')    
    navtreeProperties = getattr(propsTool, 'navtree_properties')
    
    ## Add to default_page_types
    defaultPageTypes = list(siteProperties.getProperty('default_page_types'))
    if 'EasyBlog' not in defaultPageTypes:
        defaultPageTypes.append('EasyBlog')
    siteProperties.manage_changeProperties(default_page_types = defaultPageTypes)    
        
    ## Add to metaTypesNotToList
    types = ["EasyBlogArchive", 
             "EasyBlogArchiveFolder",
             "EasyBlogCategories",
             "EasyBlogCategory",
             "EasyBlogComment",
             "EasyBlogComments",
             "EasyBlogDrafts",
             "EasyBlogEntry",
             "EasyBlogTrackback",
             "EasyBlogTrackbacks"
             ]
    metaTypesNotToList = list(navtreeProperties.getProperty('metaTypesNotToList'))
    for type in types:
        if type not in metaTypesNotToList:
            metaTypesNotToList.append(type)    
    navtreeProperties.manage_changeProperties(metaTypesNotToList = metaTypesNotToList)
        
        
    ## Add to types_not_searched
    types = ["EasyBlogCategories",
             "EasyBlogCategory",
             "EasyBlogComments",
             "EasyBlogDrafts",
             "EasyBlogTrackbacks"
             ]
    
    typesNotSearched = list(siteProperties.getProperty('types_not_searched'))
    for type in types:
        if type not in typesNotSearched:
            typesNotSearched.append(type)    
    siteProperties.manage_changeProperties(types_not_searched = typesNotSearched)
    
    ## setup product
    typeInfo = listTypes(PROJECTNAME)    
    installTypes(self, out, typeInfo, PROJECTNAME)
    install_subskin(self, out, GLOBALS)

    ## install tool
    portal = getToolByName(self, 'portal_url').getPortalObject()
    if not hasattr(self, 'easyblog_tool'):
        portal.manage_addProduct['EasyBlog'].manage_addTool(type="EasyBlogTool")
        
    ## install workflow
    wf_tool = getToolByName(self, 'portal_workflow')    

    EasyBlogWorkflow.install()    
    if not 'easyblog_workflow' in wf_tool.objectIds():
         wf_tool.manage_addWorkflow('easyblog_workflow (EasyBlog Workflow)',
                                    'easyblog_workflow')
    
    EasyBlogEntryWorkflow.install()    
    if not 'easyblogentry_workflow' in wf_tool.objectIds():
         wf_tool.manage_addWorkflow('easyblogentry_workflow (EasyBlogEntry Workflow)',
                                    'easyblogentry_workflow')
    
    EasyBlogCommentWorkflow.install()  
    if not 'easyblogcomment_workflow' in wf_tool.objectIds():
         wf_tool.manage_addWorkflow('easyblogcomment_workflow (EasyBlogComment Workflow)',
                                    'easyblogcomment_workflow')

    EasyBlogTrackbackWorkflow.install()  
    if not 'easyblogtrackback_workflow' in wf_tool.objectIds():
         wf_tool.manage_addWorkflow('easyblogtrackback_workflow (EasyBlogTrackback Workflow)',
                                    'easyblogtrackback_workflow')

    wf_tool.setChainForPortalTypes(('EasyBlog',), chain='easyblog_workflow')          
    wf_tool.setChainForPortalTypes(('EasyBlogEntry',), chain='easyblogentry_workflow')          
    wf_tool.setChainForPortalTypes(('EasyBlogComment',), chain='easyblogcomment_workflow') 
    wf_tool.setChainForPortalTypes(('EasyBlogTrackback',), chain='easyblogtrackback_workflow') 

    
    types = ["EasyBlogArchive",
             "EasyBlogArchiveFolder",             
             "EasyBlogCategory",
             "EasyBlogCategories",
             "EasyBlogTrackbacks",
             "EasyBlogComments",
             "EasyBlogDrafts",
             ]
    
    
    wf_tool.setChainForPortalTypes(types, chain='') 
    
    ## form controller    
    portal_form_controller = getToolByName(self, "portal_form_controller")    
                                              
    ## install actions
    portal_form_controller.addFormAction("validate_integrity",
                                         "success",
                                         "EasyBlogEntry",
                                         "",
                                         "traverse_to",
                                         "string:move_entry")

   
    ## install css
    csstool = getToolByName(self, 'portal_css')
    csstool.registerStylesheet('EasyBlog.css', media="all", rendering='import')

	## install js
    jstool = getToolByName(self, 'portal_javascripts')
    jstool.registerScript('easyblog.js')

    print >> out, 'Successfully installed %s' % PROJECTNAME
    return out.getvalue()

def uninstall(self):
    try:
        csstool = getToolByName(self, "portal_css")    
        csstool.unregisterResource("EasyBlog.css")
    except AttributeError:
        pass
