from Products.Archetypes.public import *

from Products.CMFCore import permissions
from Products.CMFCore.utils import getToolByName

from AccessControl import ClassSecurityInfo

from Products.EasyBlog.config import *
from Products.EasyBlog.content.EasyBlogBase import EasyBlogBase
from Products.EasyBlog.schemata.SEasyBlog import SEasyBlog
from Products.EasyBlog.interfaces.IEasyBlog import IEasyBlog
from Products.EasyBlog.interfaces.IEasyBlogBase import IEasyBlogBase
from Products.EasyBlog import EasyBlogPermissions
from Products.CMFDynamicViewFTI.browserdefault import BrowserDefaultMixin
#from Products.ATContentTypes.content.base import updateAliases
from DateTime import DateTime
from Products.CMFPlone.i18nl10n import ulocalized_time

class EasyBlog(BaseFolder, EasyBlogBase, BrowserDefaultMixin):                                                          
    """A Weblog"""

    security = ClassSecurityInfo()
    content_icon          = 'folder_icon.gif'
    meta_type             = 'EasyBlog'
    portal_type           = 'EasyBlog'
    archetype_name        = 'EasyBlog'
    immediate_view        = 'easyblog_view'
    default_view          = 'easyblog_view'
    global_allow          = True    
    filter_content_types  = True
    suppl_views = ('easyblog_view', 'easyblog_per_day_view', )
    allowed_content_types = ("Folder", )
    _at_rename_after_creation = True
    
    schema                = SEasyBlog
   
    __implements__ = (IEasyBlog, IEasyBlogBase.__implements__, BrowserDefaultMixin.__implements__)

    security.declareProtected(EasyBlogPermissions.ADD_EASYBLOG, "initializeArchetype")                
    def initializeArchetype(self, **kwargs):
        """overwritten hook"""
        BaseFolder.initializeArchetype(self, **kwargs)        

        # create portlets     
        if self.hasProperty("right_slots") == False:
            self.manage_addProperty("right_slots", ["here/portlet_review/macros/portlet",
                                                    "here/portlet_easyblog_admin/macros/portlet", 
                                                    "here/portlet_easyblog/macros/portlet", 
                                                    ], 
                                                    "lines")
    
        # Create folders the blog needs at startup
        if hasattr(self, "archive") == False:            
            self.manage_addProduct["EasyBlog"].addEasyBlogArchive(id="archive", title="Archive")
            
        if hasattr(self, "drafts") == False:
            self.manage_addProduct["EasyBlog"].addEasyBlogDrafts(id="drafts", title="Drafts")

        if hasattr(self, "categories") == False:
            self.manage_addProduct["EasyBlog"].addEasyBlogCategories(id="categories", title="Categories")
            
        if hasattr(self, "categories"):
            if hasattr(self.categories, "general") == False:
                self.categories.manage_addProduct["EasyBlog"].addEasyBlogCategory(id="general", title="General")

        # pass owner of blog the manager role
        pmt = getToolByName(self, "portal_membership")                
        self.manage_setLocalRoles(pmt.getAuthenticatedMember().getId(), ["Manager", ])
                
        # createObject (the default Plone-method to add content) needs always "Add portal content" 
        # In Order to make EasyBlog content addable by only pass "EasyBlog: Add <type>" i pass here 
        # Anonmyous "Add portal content"
        self.manage_permission(permissions.AddPortalContent, roles=["Anonymous"], acquire=True) 
                
    security.declarePublic("getCategoriesAsDicts")                
    def getCategoriesAsDicts(self):
        """Returns a list of dicts with serveral informations (id, title, amount of entries) of categories"""
        
        ebt = getToolByName(self, "easyblog_tool")
        categories_dicts = []
        for category in self.categories.objectValues():
            id = category.getId()
            title = category.Title()
            amount = ebt.getAmountOfEntries(path="/".join(self.getPhysicalPath()), category=id)
                        
            categories_dicts.append({"id" : id, "title" : title, "amount" :  amount }) 
            
        return categories_dicts

    security.declarePublic("getCategoriesAsDL")
    def getCategoriesAsDL(self):
        """Returns the categories as DefinitionList"""        
        
        categories_dl = DisplayList()        
        for category in self.categories.objectValues():
            categories_dl.add(category.id, category.title)
            
        return categories_dl
    
    security.declarePublic("getCategoryById")
    def getCategoryById(self, id):
        """Returns the category by the passed id"""     
        
        return getattr(self.categories, id, None)

    
    security.declarePublic("getPatternsToReviewTrackbacks")
    def getPatternsToReviewTrackbacks(self):
        """Returns the patterns. Resolves a possible link to review tracbacks 
           patterns
        """

        patterns = []
       
        for pattern_trackback in self.patternsToReviewTrackbacks:
            if pattern_trackback == "<patterns review comments>":
                patterns.extend(self.getPatternsToReviewComments())
            else:
                patterns.append(pattern_trackback)
                
        return patterns
                
    security.declarePublic("getPatternsToReviewComments")
    def getPatternsToReviewComments(self):
        """Returns the patterns. Resolves a possible link to review comments 
           patterns
        """

        patterns = []
        
        for pattern_comment in self.patternsToReviewComments:
            if pattern_comment == "<patterns review trackbacks>":
                patterns.extend(self.getPatternsToReviewTrackbacks())
            else:
                patterns.append(pattern_comment)

        return patterns
   
    security.declarePublic("getEntriesPerDay")
    def getEntriesPerDay(self, category="", start = DateTime("1900/01/01"), end = DateTime("2100/01/01"), path="/"):
        """Returns the blogs entries grouped by day - used in per-day-view
        
           Returns a LoL
        """

        ptool = getToolByName(self, "portal_properties").site_properties
        time_format =  ptool.getProperty("localTimeFormat")
        
        catalog = getToolByName(self, "portal_catalog")
        results = catalog.searchResults(getEntryCategories=category, 
                                        path=path, 
                                        portal_type="EasyBlogEntry", 
                                        review_state="published", 
                                        Date = {'query': [start, end], 'range': 'minmax' },
                                        sort_on='Date', 
                                        sort_order='reverse')

        
        days = []
        same_day = []
        day_before = None

        for entry in results:                                           
            entry = entry.getObject()            
            day = entry.Date().strftime("%Y%m%d")                                   
            
            if day_before == day or day_before is None:
                same_day.append(entry)
            else:
                days.append(same_day)
                same_day = []
                same_day.append(entry)
                
            day_before = day
        
        days.append(same_day)            
        return days

    security.declarePublic("getTitleString")
    def getTitleString(self):
        """Returns the title string dependend on called categoriy resp. date"""
                   
        if self.REQUEST.has_key("category"):
            category_id = self.REQUEST.get("category")
            category_name = self.getCategoryById(category_id).Title()
            return "%s - %s" % (self.Title(), category_name)
        
        elif self.REQUEST.has_key("start"):
                        
            start = DateTime(self.REQUEST.get("start"))
            end = DateTime(self.REQUEST.get("end"))
            
            if start.dd() == end.dd():                
                return "%s - %s" % (self.Title(), ulocalized_time(start, None, self))
            else:
                # translate 
                ts = self.translation_service
                month = ts.utranslate(domain="plone", msgid=ts.month_msgid(start.month()), context=self)                               
                return "%s - %s %s" % (self.Title(), month, start.year())
                
        return self.Title()
            
    security.declarePublic("canSetDefaultPage")
    def canSetDefaultPage(self):
        """Overwritten to return always false"""
        return False


    #aliases = updateAliases(
    #      BrowserDefaultMixin, 
    #      {
    #      'base_view'       : '(selected layout)',  
    #      })
        
    actions = ({'id'          : 'rss',
                'name'        : 'RSS',
                'action'      : 'string:${object_url}/easyblog_rss',
                'permissions' : (permissions.View, ),
                'category'    : 'document_actions'
                },	                    
               )     
    
registerType(EasyBlog, PROJECTNAME)
