from Products.Archetypes.public import *

from AccessControl import ClassSecurityInfo
from DateTime import DateTime

from Products.EasyBlog.config import *
from Products.EasyBlog.content.EasyBlogBase import EasyBlogBase
from Products.EasyBlog.schemata.SEasyBlogArchive import SEasyBlogArchive
from Products.EasyBlog.interfaces.IEasyBlogArchive import IEasyBlogArchive
from Products.EasyBlog.interfaces.IEasyBlogBase import IEasyBlogBase
from Products.CMFCore.utils import getToolByName

class EasyBlogArchive(BaseFolder, EasyBlogBase):                                                          
    """Archives published Entries in folders dependend on the creation date"""

    security = ClassSecurityInfo()
    content_icon          = 'folder_icon.gif'
    meta_type             = 'EasyBlogArchive'
    portal_type           = 'EasyBlogArchive'
    archetype_name        = 'EasyBlogArchive'
    immediate_view        = 'easyblogarchive_view'
    default_view          = 'easyblogarchive_view'
    global_allow          = False    
    filter_content_types  = True
    allowed_content_types = ("EasyBlogArchiveFolder", )    
    schema                = SEasyBlogArchive
        
    __implements__ = (IEasyBlogBase.__implements__, IEasyBlogArchive)

    security.declarePublic("moveEntry")
    def moveEntry(self, entry):
        """Cut 'n pastes the given entry to the archive (creates the needed date folders)"""

        date = DateTime(entry.Date())
        
        year = str(date.year())          
        month = str(date.mm())
        day = str(date.dd())
        entry_id = entry.getId()
         
        # create the folder out of entrys date
        # Use try/except here because otherwise (with hasattr) there are 
        # problems if month == day, e.g. 10.10.2005
        
        try:
            self.manage_addProduct["EasyBlog"].addEasyBlogArchiveFolder(id=year, title=year)
        except:
            pass
            
        year_folder = self[year]
        
        try:
            year_folder.manage_addProduct["EasyBlog"].addEasyBlogArchiveFolder(id=month, title=month)
        except:
            pass
            
        month_folder = year_folder[month]
        
        try:
            month_folder.manage_addProduct["EasyBlog"].addEasyBlogArchiveFolder(id=day, title=day)
        except:
            pass
    
        day_folder = month_folder[day]
        
        parent_folder = entry.aq_inner.aq_parent
        
        # get
        entry = parent_folder._getOb(entry_id)        
        
        # copy 
        day_folder._setObject(entry_id, entry)
        
        # delete
        parent_folder._delObject(entry_id)

        ## del folders
        self.delEmptyFolders(parent_folder)

        entry = day_folder._getOb(entry_id) 
        
        ## redirect to new location (in calling script)                
        return entry

                             
    security.declarePublic("getArchivesAsDicts")    
    def getArchivesAsDicts(self):
        """Returns all monthly archives as a dict of strings"""
                
        ebt = getToolByName(self, "easyblog_tool")
        catalog = getToolByName(self, "portal_catalog")
        months = []
        for year in self.objectValues():
            for month in year.objectValues():
                start = "%s/%s/01" % (year.getId(), month.getId())
                end = "%s/%s/%s" % (year.getId(), month.getId(), ebt.getUltimoFor(year.getId(), month.getId()))
                
                # translate month
                ts = self.translation_service
                month_str = ts.utranslate(domain="plone", msgid=ts.month_msgid(month.getId()), context=self)                               
                year_str = year.getId()
                
                amount = ebt.getAmountOfEntries(path="/".join(self.getPhysicalPath()), start = DateTime(start), end = DateTime(end) )
                
                months.append({ "start": start, "end" : end, "month": month_str, "year": year_str, "amount" : amount })
        
        return months
    
    security.declarePrivate("delEmptyFolders")
    def delEmptyFolders(self, start_folder):
        """Deletes empty folders from a given start folder up to archive"""
        
        if start_folder.getId() == "drafts":
            return 
        
        while start_folder.portal_type != "EasyBlogArchive":
            if len(start_folder.objectIds())==0:
                parent_tmp = start_folder.aq_inner.aq_parent
                parent_tmp._delObject(start_folder.id)
            start_folder = start_folder.aq_inner.aq_parent


    aliases = ({'(Default)' : 'easyblogarchive_view', 
                'view'      : 'easyblogarchive_view',
                'base_view' : 'easyblogarchive_view', 
                })

def modify_fti(fti):
    for a in fti['actions']:
        if a['id'] in ('edit', 'references', 'metadata'):
            a['visible'] = 0
    return fti
    
registerType(EasyBlogArchive, PROJECTNAME)
