from Products.Archetypes.public import *

from Products.CMFCore import permissions

from AccessControl import ClassSecurityInfo

from Products.EasyBlog.config import *
from Products.EasyBlog.content.EasyBlogBase import EasyBlogBase
from Products.EasyBlog.schemata.SEasyBlogComment import SEasyBlogComment
from Products.EasyBlog.interfaces.IEasyBlogComment import IEasyBlogComment
from Products.EasyBlog.interfaces.IEasyBlogBase import IEasyBlogBase
from Products.EasyBlog.utils.CommentHTMLParser import CommentHTMLParser

from Products.CMFCore.utils import getToolByName 
import re 

class EasyBlogComment(BaseContent, EasyBlogBase):                                                          
    """A comment to a entry"""

    security = ClassSecurityInfo()
    content_icon          = 'document_icon.gif'
    meta_type             = 'EasyBlogComment'
    portal_type           = 'EasyBlogComment'
    archetype_name        = 'EasyBlogComment'
    immediate_view        = 'easyblogcomment_view'
    default_view          = 'easyblogcomment_view'
    global_allow          = False    
    schema                = SEasyBlogComment
        
    schema['title'].widget.label = 'Name'
    schema['title'].widget.description ='Your name.'
    
    # Breaks breadcrumbs in Plone 2.0.5
    __implements__ = (IEasyBlogBase.__implements__, IEasyBlogComment)
    
    security.declarePublic('setIpAddress')
    def setIpaddress(self, dummy):
        """Sets the IPAddress"""        
        self.ipaddress = self.REQUEST.get('HTTP_X_FORWARDED_FOR', self.REQUEST.get('REMOTE_ADDR', ''))

    security.declarePublic('getMessage')
    def getMessage(self):
        """Returns the message with forbidden tags and attributes translated to plain text"""

        # use *get*Message (not set) here to save the origin message
        # so it's possible later to change the allowed tags
        
        msg = str(self.message)
        msg = msg.replace('\n', '<br/>')
   
        #return msg
        # parse for (un)allowed tags
        blog = self.getBlog()
        parser = CommentHTMLParser(blog.getHtmlTags(), 
                                   blog.getHtmlTagsHandling(),
                                   blog.getHtmlAttributes(),
                                   blog.getHtmlAttributesHandling()
                                   )        
        parser.feed(msg)
        
        return parser.getHTML()

    security.declarePublic("alwaysReview")
    def alwaysReview(self):
        """Returns true if the comment should always moderated"""
        
        patterns = self.getBlog().getPatternsToReviewComments()
        text = "%s %s %s %s" % (self.title, self.getEmail(), self.getHomepage(), self.getMessage()) 
        
        for pattern in patterns:
            mo = re.search(pattern, text)
            if mo is not None:
                return True
        
        return False

    security.declarePublic('easyblogcomment_view')
    def easyblogcomment_view(self):
        """My View"""        
        # redirect to blog view
        pm = getToolByName(self, "portal_membership")

        if pm.checkPermission(permissions.ManagePortal, self) is None:                             
            return self.aq_parent.aq_parent.easyblogentry_view()
        else:
            return self.base_view()
        
    security.declareProtected("EasyBlog: Delete EasyBlogComment", "delete")    
    def delete(self):
        """delete this comment"""
        
        # Use own delete method get rid of "Delete objects"-permission,
        # which has a scope of the whole parent folder and a draft should 
        # only be deleteable by its owner.
        
        parent = self.aq_inner.aq_parent
        parent._delObject(self.id)
        
        parent = parent.aq_inner.aq_parent
        return self.REQUEST.RESPONSE.redirect(
                    "%s?portal_status_message=%s" % 
                    (parent.absolute_url(), 
                    "Comment has been deleted."))
    
    aliases = ({'(Default)' : 'easyblogcomment_view', 
                'view'      : 'easyblogcomment_view',
                'base_view' : 'easyblogcomment_view',
                'base_edit' : 'easyblogcomment_edit_form',                
                })
 
def modify_fti(fti):
    for a in fti['actions']:
        if a['id'] in ('references', 'metadata'):
            a['visible'] = 0
    return fti
    
registerType(EasyBlogComment, PROJECTNAME)
