from Products.Archetypes.public import *

from Products.CMFCore import permissions
from Products.CMFPlone.interfaces.NonStructuralFolder import INonStructuralFolder

from AccessControl import ClassSecurityInfo

from Products.EasyBlog.config import *
from Products.EasyBlog.content.EasyBlogBase import EasyBlogBase
from Products.EasyBlog.schemata.SEasyBlogEntry import SEasyBlogEntry
from Products.EasyBlog.interfaces.IEasyBlogEntry import IEasyBlogEntry
from Products.EasyBlog.interfaces.IEasyBlogBase import IEasyBlogBase
from Products.EasyBlog.utils.BodyHTMLParser import BodyHTMLParser
from Products.EasyBlog.utils.TrackbackManager import TrackbackManager
from Products.EasyBlog.utils.EasyLogger import EasyLogger
from Products.CMFCore.utils import getToolByName
from DateTime import DateTime

class EasyBlogEntry(BaseFolder, EasyBlogBase):                                                          
    """Represents a entry within EasyBlog"""

    # XXX: think about this (make it sense to be a class attribute?)
    trackback_manager = TrackbackManager()
    
    security = ClassSecurityInfo()
    content_icon          = 'document_icon.gif'
    meta_type             = 'EasyBlogEntry'
    portal_type           = 'EasyBlogEntry'
    archetype_name        = 'EasyBlogEntry'
    immediate_view        = 'view'
    default_view          = 'view'   
    suppl_views           = ()
    global_allow          = False
    filter_content_types  = True
    allowed_content_types = ("EasyBlogTrackbacks", "EasyBlogComments")    
    _at_rename_after_creation = True
    schema                = SEasyBlogEntry

    __implements__ = (IEasyBlogBase.__implements__, IEasyBlogEntry, INonStructuralFolder)

    def __init__(self, oid, **kwargs):
        """My Constructor"""
        BaseFolder.__init__(self, oid, **kwargs)

        self.trackbackLog = EasyLogger()                

    security.declarePublic("initializeArchetype")
    def initializeArchetype(self, **kwargs):
        """overwritten hook"""
        BaseFolder.initializeArchetype(self, **kwargs)        
        
        # add trackbacks folder
        if hasattr(self, "trackbacks") == False:            
            self.manage_addProduct["EasyBlog"].addEasyBlogTrackbacks(id="trackbacks", title="Trackbacks")

        if hasattr(self, "comments") == False:            
            self.manage_addProduct["EasyBlog"].addEasyBlogComments(id="comments", title="Comments")
            
    security.declarePublic('moveToDraft')    
    def moveToDraft(self, sc):
        """ """
        pmt = getToolByName(self, "portal_membership")
        
        entry_id = self.getId()        
        drafts_folder = self.getBlog().drafts
        parent_folder = self.aq_inner.aq_parent

        # just move to draft if published (not pending)
        # if status is pending is anyway in draft        
        if sc.old_state.getId() == "published":    
                        
            # get
            entry = parent_folder._getOb(entry_id)                               

            # copy 
            drafts_folder._setObject(entry_id, entry)

            # delete
            parent_folder._delObject(entry_id)            

            ## del archive folders
            while parent_folder.portal_type != "EasyBlogArchive":
                if len(parent_folder.objectIds())==0:
                    parent_tmp = parent_folder.aq_inner.aq_parent
                    parent_tmp._delObject(parent_folder.id)
                parent_folder = parent_folder.aq_inner.aq_parent
            
        ## redirect to the now moved object            
        entry = drafts_folder[entry_id] 

        if pmt.checkPermission("View", entry):
           raise sc.ObjectMoved(entry, entry)
        else:
           raise sc.ObjectMoved(entry, entry.getBlog())

    security.declarePublic('getDefaultForAutodetectTrackbacks')    
    def getDefaultForAutodetectTrackbacks(self):
        """Get the default value for autodetect trackbacks from the blog
        """
        return self.getBlog().getAutodetectTrackbacks()
        
    # name it showComments to let standard accessor untouched (for edit form)    
    security.declarePublic('showComments')
    def showCommentsInView(self):
        """Returns the value for show comments"""        
        
        if self.showComments == "default":
            return self.getBlog().getShowComments()
        elif self.showComments == "yes":
            return True
        else:
            return False
        
    security.declarePublic('allowComments')
    def allowCommentsInView(self):
        """Returns the value for allow comments"""        
        
        if self.allowComments == "default":
            return self.getBlog().getAllowComments()
        elif self.allowComments == "yes":
            return True
        else:
            return False
            
    security.declarePublic('showTrackbacksInView')
    def showTrackbacksInView(self):
        """Returns the value for show trackbacks"""                
        if self.showTrackbacks == "default":
            return self.getBlog().getShowTrackbacks()
        elif self.showTrackbacks == "yes":
            return True
        else:
            return False

    security.declarePublic('receiveTrackbacksInView')
    def receiveTrackbacksInView(self):
        """Returns the value for receive comments"""        
        
        if self.receiveTrackbacks == "default":
            return self.getBlog().getReceiveTrackbacks()
        elif self.receiveTrackbacks == "yes":
            return True
        else:
            return False
            
    security.declarePublic('getTrackbackLog')
    def getTrackbackLog(self):
        """Returns an EasyLogger"""
        return self.trackbackLog

    security.declarePublic("content_type")
    def content_type(self):
        """Overwritten to provide the respective content_type (text/more)"""
        field = self.REQUEST.get('field', 'text')
        return self.getContentType(field)
        
    security.declareProtected(permissions.ModifyPortalContent, 'PUT')
    def PUT(self, REQUEST, RESPONSE):
        """Overwritten to save text/more field"""

        # get which field was edited
        field = self.getProperty('field')

        # Save content_type for later use
        content_type = self.getContentType(field)

        self.dav__init(REQUEST, RESPONSE)
        self.dav__simpleifhandler(REQUEST, RESPONSE, refresh=1)
        body = REQUEST.get('BODY', '')
        
        # send it through parser if it HTML
        if REQUEST.get_header('Content-Type', '') == 'text/html':
            parser = BodyHTMLParser()
            parser.feed(body)
            body = parser.html
            
        if field == "text":
            self.setText(body)
        else:
            self.setMore(body)
        
        # overwrite the standard setContentType-process 
        # which seems to be automatically called via standard accessor
        # which make problems if no tags are inputed in NVU for example.
        self.setContentType(content_type, field)
        
        # Finally reindex to update the catalog
        self.reindexObject()

    security.declareProtected(permissions.ModifyPortalContent, 'manage_FTPget')    
    def manage_FTPget(self, REQUEST, RESPONSE):
        """Overwritten to get the text/more field"""
               
        field = REQUEST.get('field', 'text')
        
        if self.hasProperty("field"):
            self.manage_delProperties(["field",])
                    
        self.manage_addProperty("field", field, "string")
        
        content_type = self.getContentType(field)
        
        text = self.getField(field).get(self, content_type)
        return text
        
    security.declarePublic('updateDate')
    def setUpdateDate(self, dummy):
        """Sets update date always to 'no_update'"""
        self.updateDate = 'no_update'

    security.declarePublic('setEntryDate')
    def setEntryDate(self, date):
        """Sets the date dependend on the user's selection"""                
       
        if self.REQUEST.get("updateDate") == "update_to_now":
            now = DateTime()
            self.entryDate = now
        elif self.REQUEST.get("updateDate") == "update_to_date":
            self.entryDate = DateTime(date)
                
    security.declarePublic("Date")    
    def Date(self):
        """Returns the date which is responsible for sorting and calendaring"""
        # XXX: Need this to return the object rather than a string (?)        
        
        # Have to take the function name "Date" (not getDate, ...) here, 
        # because in this way the standard index of the portal_catalog is working
        
        return self.getEntryDate()
        
    security.declarePublic("start")
    def start(self):
        """Start date of the entry, needed for portal_calendar"""
        return self.Date()
    
    security.declarePublic("end")
    def end(self):
        """End date of the entry (same as start), needed for portal_calendar"""
        return self.Date()
    
    security.declarePublic("getAmountOfComments")
    def getAmountOfComments(self):
        """Get the amount of comments which are associated with this entry"""
        
        portal_catalog = getToolByName(self, "portal_catalog")        
        comments = portal_catalog.searchResults(portal_type="EasyBlogComment", review_state="published", path="/".join(self.getPhysicalPath()))
               
        return len(comments)
    
    security.declarePublic("getAmountOfTrackbacks")
    def getAmountOfTrackbacks(self):
        """Get the amount of trackback which are associated with this entry"""
                
        portal_catalog = getToolByName(self, "portal_catalog")        
        trackbacks = portal_catalog.searchResults(portal_type="EasyBlogTrackback", review_state="published", path="/".join(self.getPhysicalPath()))

        return len(trackbacks)
      
    security.declarePublic("getEntryCategories")
    def getEntryCategories(self):
        """returns only categories which still exists"""
        categories = []
        
        if hasattr(self, "entryCategories") == False:
            return categories
                
        for category in self.entryCategories:
            if self.getBlog().getCategoryById(category) is not None:
                categories.append(category)
    
        return categories

    security.declarePublic("sendTrackbacks")
    def sendTrackbacks(self):
        """Delegates sending trackbacks to TrackbackManager"""
                
        self.trackback_manager.sendTrackbacks(self)

    security.declareProtected("EasyBlog: Delete EasyBlogEntry", "delete")    
    def delete(self):
        """delete this entry"""
        
        # Use own delete method get rid of "Delete objects"-permission,
        # which has a scope of the whole parent folder and a draft should 
        # only be deleteable by its owner.
        
        parent = self.aq_inner.aq_parent
        parent._delObject(self.id)
        
        return self.REQUEST.RESPONSE.redirect(
                    "%s?portal_status_message=%s %s" % 
                    (self.getBlog().absolute_url(), 
                    self.Title(), 
                    "has been deleted."))
    
    def view(self):
        """Decides whether it's a request or a trackback"""
                
        if self.REQUEST["REQUEST_METHOD"] == "POST" and self.REQUEST.has_key("url"):
            if self.receiveTrackbacksInView() == True:
                self.trackback_manager.addTrackback(self)
            else:
                if self.getBlog().getReceiveTrackbacks() == False:
                    msg = "Sorry, trackbacks are not supported for this blog."
                else:
                    msg = "Sorry, trackbacks are not supported for this entry."                    
                self.REQUEST.RESPONSE.write(self.trackback_manager._getResponseMessage(1, msg))    
        else:
            return self.easyblogentry_view()
    
    aliases = ({'(Default)' : 'view', 
                'view'      : 'view',
                'base_view' : 'view', 
                })
    
registerType(EasyBlogEntry, PROJECTNAME)
