from Products.Archetypes.public import *
from AccessControl import ClassSecurityInfo
from Products.EasyBlog.config import *
from Products.EasyBlog.content.EasyBlogBase import EasyBlogBase
from Products.EasyBlog.schemata.SEasyBlogTrackback import SEasyBlogTrackback
from Products.EasyBlog.interfaces.IEasyBlogTrackback import IEasyBlogTrackback
from Products.EasyBlog.interfaces.IEasyBlogBase import IEasyBlogBase
from Products.EasyBlog.utils.ExcerptHTMLParser import ExcerptHTMLParser
import re 


class EasyBlogTrackback(BaseContent, EasyBlogBase):                                                          
    """A comment to a entry sent via a trackback"""

    security = ClassSecurityInfo()
    content_icon          = 'document_icon.gif'
    meta_type             = 'EasyBlogTrackback'
    portal_type           = 'EasyBlogTrackback'
    archetype_name        = 'EasyBlogTrackback'
    immediate_view        = 'base_view'
    default_view          = 'base_view'
    global_allow          = False    
    schema                = SEasyBlogTrackback
    
    # Breaks breadcrumbs in Plone 2.0.5
    __implements__ = (IEasyBlogBase.__implements__, IEasyBlogTrackback)

    security.declarePublic('setIpAddress')
    def setIpaddress(self, dummy):
        """Sets the IPAddress"""        
        self.ipaddress = self.REQUEST.get('HTTP_X_FORWARDED_FOR', self.REQUEST.get('REMOTE_ADDR', ''))

    # cut excerpt here - and save the whole - to be able to change the mind later on
    # XXX: Profile it!
    security.declarePublic("getExcerpt")    
    def getExcerpt(self):        
        """Accessor of Excerpt"""
        max_chars = int(self.getBlog().getMaxCharsInTrackbacks())
        excerpt = str(self.excerpt)
        
        if max_chars > 0:       

            p = ExcerptHTMLParser(max_chars)
            p.feed(excerpt)         
            part_excerpt = p.data
            
            if len(part_excerpt) < len(excerpt):
                part_excerpt += """ <a title="%s" href="%s">More</a>""" % (self.Title(), self.getUrl())
            
            return part_excerpt
                    
        return excerpt

    security.declarePublic("alwaysReview")
    def alwaysReview(self):
        """Returns true if the trackback should always moderated"""

        patterns = self.getBlog().getPatternsToReviewTrackbacks()
        
        # take the attribute excerpt here, to check the whole excerpt
        text = "%s %s %s" % (self.getBlogname(), self.getUrl(), self.excerpt) 
        
        for pattern in patterns:
            mo = re.search(pattern, text)
            if mo is not None:
                return True
        
        return False

    security.declareProtected("EasyBlog: Delete EasyBlogTrackback", "delete")    
    def delete(self):
        """delete this trackback"""
        
        # Use own delete method get rid of "Delete objects"-permission,
        # which has a scope of the whole parent folder and a draft should 
        # only be deleteable by its owner.
        
        parent = self.aq_inner.aq_parent
        parent._delObject(self.id)
        
        parent = parent.aq_inner.aq_parent
        return self.REQUEST.RESPONSE.redirect(
                    "%s?portal_status_message=%s" % 
                    (parent.absolute_url(), 
                    "Trackback has been deleted."))
                   
def modify_fti(fti):
    for a in fti['actions']:
        if a['id'] in ('references', 'metadata'):
            a['visible'] = 0
    return fti
    
registerType(EasyBlogTrackback, PROJECTNAME)

