## Test EasyBlog

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.EasyBlog.tests import EasyBlogTestCase
try:
    from Products.LinguaPlone.public import *
except ImportError:
    # No multilingual support
    from Products.Archetypes.public import *

from Products.CMFCore.utils import transaction
from Products.CMFCore.WorkflowCore import WorkflowException
from DateTime import DateTime
from AccessControl import Unauthorized

class TestEasyBlog(EasyBlogTestCase.EasyBlogTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.folder.invokeFactory("EasyBlog", id="blog", title="Blog", description="Blog Description")
        self.blog = self.folder.blog
        
    def testTypeInformation(self):               
        portal_type = self.portal.portal_types        
        type_info = portal_type.getTypeInfo("EasyBlog")
        
        self.assertEqual(type_info.global_allow, True)
        self.assertEqual(type_info.filter_content_types, True)
        self.assertEqual(type_info.allowed_content_types, ("Folder", ))
        self.assertEqual(type_info.content_meta_type, "EasyBlog")

        ## actions
        actions = [a.id for a in type_info._actions]        
        self.assertEqual(actions, ['view', 'edit', 'metadata', 'references', 'folderlisting', 'rss'])
     
    def testInvoking(self):       
        blog = self.blog
        self.assertEqual(blog.getId(), "blog")
        self.assertEqual(blog.Title(), "Blog")
        self.assertEqual(blog.getDescription(), "Blog Description")
        self.assertEqual(blog.getPatternsToReviewComments(), [])
        self.assertEqual(blog.getPatternsToReviewTrackbacks(), [])
        self.assertEqual(blog.patternsToReviewTrackbacks, (u"<patterns review comments>", ))
        
        ## should be automatically created 
        self.assertEqual(blog.drafts.portal_type, "EasyBlogDrafts")
        self.assertEqual(blog.archive.portal_type, "EasyBlogArchive")
        self.assertEqual(blog.categories.portal_type, "EasyBlogCategories")
        
    def testAddCategory(self):              
        blog = self.blog
        ## add a new category
        blog.categories.invokeFactory("EasyBlogCategory", id="ploning", title="Ploning")
        
        ## get all category-ids 
        categories = [category.getId() for category in blog.categories.objectValues()]        
        self.assertEqual(categories, ['general', 'ploning'])

    def testGetCategoriesAsDicts(self):
        blog = self.blog
        blog.categories.invokeFactory("EasyBlogCategory", id="ploning", title="Ploning")
        blog.categories.invokeFactory("EasyBlogCategory", id="zoping", title="Zoping")
        
        dicts = [{"id":"general", "title":"General", "amount":0},
                 {"id":"ploning", "title":"Ploning", "amount":0},
                 {"id":"zoping", "title":"Zoping", "amount":0}]
        
        self.assertEqual(blog.getCategoriesAsDicts(), dicts)

    def testGetCategoriesAsDL(self):
        blog = self.blog
        blog.categories.invokeFactory("EasyBlogCategory", id="ploning", title="Ploning")
        blog.categories.invokeFactory("EasyBlogCategory", id="zoping", title="Zoping")        
        
        blog.getCategoriesAsDL()
        dl = DisplayList()
        dl.add('general', 'General')
        dl.add('ploning', 'Ploning')
        dl.add('zoping', 'Zoping')

        self.assertEqual(blog.getCategoriesAsDL(), dl)
                   
    def testGetCategoryById(self):            
        blog = self.blog
        category = blog.getCategoryById('general')
        self.assertEqual(category.Title(), 'General')

    def testGetPatternsToReviewTrackbacks(self):
        blog = self.blog
        blog.setPatternsToReviewTrackbacks(["one", "two"])
        blog.setPatternsToReviewComments(["three", "four"])        
        
        # w/o patterns from comments
        self.assertEqual(blog.getPatternsToReviewTrackbacks(), ["one", "two"])

        # get patterns from comments
        blog.setPatternsToReviewTrackbacks(["one", "two", "<patterns review comments>"])
        self.assertEqual(blog.getPatternsToReviewTrackbacks(), ["one", "two", "three", "four"])
        
    def testGetPatternsToReviewComments(self):
        blog = self.blog
        blog.setPatternsToReviewTrackbacks(["one", "two"])
        blog.setPatternsToReviewComments(["three", "four"])        
        
        # w/o patterns from comments
        self.assertEqual(blog.getPatternsToReviewComments(), ["three", "four"])

        # get patterns from comments
        blog.setPatternsToReviewComments(["three", "four", "<patterns review trackbacks>"])
        self.assertEqual(blog.getPatternsToReviewComments(), ["three", "four", "one", "two"])
    
    def testGetEntriesPerDay(self):
        
        workflow = self.portal.portal_workflow
        
        blog = self.blog 
        drafts = blog.drafts
        
        # need updateDate to set date manually (s. blog.setEntryDate())
        blog.REQUEST.set("updateDate", "update_to_date")
        
        # add some entries               
        blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry_1", title="Blogentry_1", entryDate="2005-10-20")           
        blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry_2", title="Blogentry_2", entryDate="2005-10-20")        
        blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry_3", title="Blogentry_3", entryDate="2005-10-21")        
        blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry_4", title="Blogentry_4", entryDate="2005-10-21")        
        blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry_5", title="Blogentry_5", entryDate="2005-10-22")                
        transaction.commit(1)             

        # publish my entries as getEntriesPerDay returns just published entries                
        for i in range(1, 6):        
            workflow.doActionFor(drafts["blogentry_%s" % i], "publish")
        
        # get it 
        epd = blog.getEntriesPerDay()
        
        # test it
        self.assertEqual(epd["2005-10-20"][0].id, "blogentry_1")
        self.assertEqual(epd["2005-10-20"][1].id, "blogentry_2")        
        self.assertEqual(epd["2005-10-21"][0].id, "blogentry_3")
        self.assertEqual(epd["2005-10-21"][1].id, "blogentry_4")
        self.assertEqual(epd["2005-10-22"][0].id, "blogentry_5")
        
    def testGetTitleString(self):
        
        blog = self.blog 
        
        # case 1: without anything
        title = blog.getTitleString()
        self.assertEqual("Blog", title)

        # case 2: with start == end
        blog.REQUEST.set("start", "2005-10-20")
        blog.REQUEST.set("end", "2005-10-20")
        title = blog.getTitleString()
        self.assertEqual("Blog - 2005-10-20", title)

        # case 3: with start != end         
        blog.REQUEST.set("start", "2005-10-20")
        blog.REQUEST.set("end", "2005-10-21")
        title = blog.getTitleString()
        self.assertEqual("Blog - month_oct 2005", title)
        
        # case 4: with category
        blog.REQUEST.set("category", "general")
        title = blog.getTitleString()
        self.assertEqual("Blog - General", title)        
        

class TestEasyBlogSecurityViewBlog(EasyBlogTestCase.EasyBlogTestCase):

    def afterSetUp(self):
        
        self.pwt = self.portal.portal_workflow
        
        # set permissions, so that a member can create a easyblog
        self.folder.manage_permission('EasyBlog: Add EasyBlog', ['Member'], acquire=1)
        
        self.membership = self.portal.portal_membership
        self.membership.addMember('owner', 'secret', ['Member'], [])                
        self.membership.addMember('member', 'secret', ['Member'], [])        
        self.membership.addMember('manager', 'secret', ['Manager', 'Member'], [])        
        
        self.login("owner")
        self.folder.invokeFactory("EasyBlog", id="blog", title="Blog", description="Blog Description")
        self.blog = self.folder.blog
        self.logout()
        
    def testViewBlogPrivate(self):
                        
        blog = self.blog 
        
        # anon
        self.assertRaises(Unauthorized, self.folder.restrictedTraverse, "blog")
        
        # member
        self.login("member")        
        self.assertRaises(Unauthorized, self.folder.restrictedTraverse, "blog")        
        
        # owner 
        self.login("owner")        
        self.folder.restrictedTraverse("blog")        
        
        # manager
        self.login("manager")        
        self.folder.restrictedTraverse("blog")                

    def testViewBlogClosed(self):
                        
        blog = self.blog
        self.login("owner")
        self.pwt.doActionFor(blog, "make_closed")
        self.logout()
        
        # anon
        self.assertRaises(Unauthorized, self.folder.restrictedTraverse, "blog")
        
        # member
        self.login("member")        
        self.folder.restrictedTraverse("blog")        
        
        # owner 
        self.login("owner")        
        self.folder.restrictedTraverse("blog")        
        
        # manager
        self.login("manager")        
        self.folder.restrictedTraverse("blog")                

    def testViewBlogPublic(self):
                        
        blog = self.blog 
        self.login("owner")
        self.pwt.doActionFor(blog, "make_public")
        self.logout()
        
        # anon
        self.folder.restrictedTraverse("blog") 
        
        # member
        self.login("member")        
        self.folder.restrictedTraverse("blog")        
        
        # owner 
        self.login("owner")        
        self.folder.restrictedTraverse("blog")        
        
        # manager
        self.login("manager")        
        self.folder.restrictedTraverse("blog")                
        
class TestEasyBlogSecurityReviewBlog(EasyBlogTestCase.EasyBlogTestCase):

    def afterSetUp(self):
        
        self.pwt = self.portal.portal_workflow
        
        # set permissions, so that a member can create a easyblog
        self.folder.manage_permission('EasyBlog: Add EasyBlog', ['Member'], acquire=1)
        
        self.membership = self.portal.portal_membership
        self.membership.addMember('owner', 'secret', ['Member'], [])                
        self.membership.addMember('member', 'secret', ['Member'], [])        
        self.membership.addMember('manager', 'secret', ['Manager', 'Member'], [])        
        self.membership.addMember('reviewer', 'secret', ['Manager', 'Reviewer'], [])        
        
        self.login("owner")
        self.folder.invokeFactory("EasyBlog", id="blog", title="Blog", description="Blog Description")
        self.pwt.doActionFor(self.folder.blog, "make_public")
        
        self.folder.blog.drafts.invokeFactory("EasyBlogEntry", id="entry_1")
        self.folder.blog.drafts.invokeFactory("EasyBlogEntry", id="entry_2")        
        self.folder.blog.drafts.invokeFactory("EasyBlogEntry", id="entry_3")                
        self.folder.blog.drafts.invokeFactory("EasyBlogEntry", id="entry_4")         
        self.blog = self.folder.blog

        self.entry_1 = self.blog.drafts.entry_1
        self.entry_2 = self.blog.drafts.entry_2        
        self.entry_3 = self.blog.drafts.entry_3                
        self.entry_4 = self.blog.drafts.entry_4                        
        self.logout()
    
        # to make move_to_archive possible
        transaction.commit(1)  
        
    def testReviewBlog(self):
        
        # no one means: no member, no anon. owner and manger are allowed.
        
        blog = self.blog 
        
        entry_1 = self.entry_1
        entry_2 = self.entry_2        
        entry_3 = self.entry_3
        entry_4 = self.entry_4        
        
        # a1) member not allowed
        self.login("member")
        self.assertRaises(Unauthorized, self.pwt.doActionFor, entry_1, "publish")

        self.login("manager")
        entry_1.manage_role("Member", ["View", "EasyBlog: Review Blog", "Access contents information"])
        
        # a2) member allowed
        self.login("member")
        self.pwt.doActionFor(entry_1, "publish")
                
        # b) reviewer
        self.login("reviewer")                
        self.pwt.doActionFor(entry_2, "publish")
        
        # c) owner
        self.login("owner")
        self.pwt.doActionFor(entry_3, "publish")

        # d) manager
        self.login("manager")
        self.pwt.doActionFor(entry_4, "publish")

        
class TestEasyBlogSecurityAddEntry(EasyBlogTestCase.EasyBlogTestCase):

    def afterSetUp(self):
        
        # set permissions, so that a member can create a easyblog
        self.folder.manage_permission('EasyBlog: Add EasyBlog', ['Member'], acquire=1)
        
        self.membership = self.portal.portal_membership
        self.membership.addMember('owner', 'secret', ['Member'], [])                
        self.membership.addMember('member', 'secret', ['Member'], [])        
        self.membership.addMember('manager', 'secret', ['Manager', 'Member'], [])        
        
        self.login("owner")
        self.folder.invokeFactory("EasyBlog", id="blog", title="Blog", description="Blog Description")
        self.blog = self.folder.blog
        self.logout()

           
    def testAddEntryNoOneAllowed(self):
        
        blog = self.blog
        
        # case 1: no one is allowed                
        
        # a) anon         
        self.assertRaises(Unauthorized, blog.drafts.invokeFactory, 'EasyBlogEntry', id="blogentry_1", title="Blogentry_1")

        # b) member
        self.login("member")
        self.assertRaises(Unauthorized, blog.drafts.invokeFactory, 'EasyBlogEntry', id="blogentry_2", title="Blogentry_2")

        # c) owner 
        self.login("owner")
        blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry_3", title="Blogentry_3")
        
        # c) manager
        self.login("manager")
        blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry_4", title="Blogentry_4")


    def testAddEntryMembersAllowed(self):

        blog = self.blog        
        blog.manage_permission("EasyBlog: Add EasyBlogEntry", ["Member",])
        
        # a) anon
        self.assertRaises(Unauthorized, blog.drafts.invokeFactory, 'EasyBlogEntry', id="blogentry_1", title="Blogentry_1")

        # b) member
        self.login("member")
        blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry_2", title="Blogentry_2")

        # c) owner 
        self.login("owner")
        blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry_3", title="Blogentry_3")
        
        # c) manager
        self.login("manager")
        blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry_4", title="Blogentry_4")
        
    def testAddEntryAllAllowed(self):

        blog = self.blog            
        blog.manage_permission("EasyBlog: Add EasyBlogEntry", ["Anonymous", "Member"])
        
        # a) anon
        blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry_1", title="Blogentry_1")

        # b) member
        self.login("member")
        blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry_2", title="Blogentry_2")

        # c) owner 
        self.login("owner")
        blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry_3", title="Blogentry_3")
        
        # c) manager
        self.login("manager")
        blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry_4", title="Blogentry_4")
        
        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestEasyBlog))
    suite.addTest(makeSuite(TestEasyBlogSecurityAddEntry))
    suite.addTest(makeSuite(TestEasyBlogSecurityReviewBlog))    
    suite.addTest(makeSuite(TestEasyBlogSecurityViewBlog))        
    return suite

if __name__ == '__main__':
    framework()

