## Test EasyBlogArchive

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.EasyBlog.tests import EasyBlogTestCase
from DateTime import DateTime
from Products.CMFCore.utils import transaction

class TestEasyBlogArchive(EasyBlogTestCase.EasyBlogTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager'])
        self.folder.invokeFactory("EasyBlog", id="blog", title="Blog", description="Blog Description")
        
        self.blog = self.folder.blog
        self.drafts = self.folder.blog.drafts
        self.archive = self.folder.blog.archive
        
    def testTypeInformation(self):               
        portal_type = self.portal.portal_types        
        type_info = portal_type.getTypeInfo("EasyBlogArchive")
        
        self.assertEqual(type_info.global_allow, False)
        self.assertEqual(type_info.filter_content_types, True)
        self.assertEqual(type_info.allowed_content_types, ("EasyBlogArchiveFolder", ))
        self.assertEqual(type_info.content_meta_type, "EasyBlogArchive")

        ## actions
        actions = [a.id for a in type_info._actions]        
        self.assertEqual(actions, ['view', 'edit', 'metadata', 'references', 'folderlisting'])
     
    def testInvoking(self):       
        archive = self.archive        
        self.assertEqual(archive.getId(), "archive")
        self.assertEqual(archive.Title(), "Archive")
            
    def testMoveEntryFromDrafts(self):
        drafts = self.drafts
        archive = self.archive
        blog = self.blog
                
        ## create entry
        drafts.invokeFactory("EasyBlogEntry", id="blogentry", title="BlogEntry")
        
        ## commit to entry make movable
        transaction.commit(1)             

        blogentry = drafts.blogentry
        
        ## test moveEntry
        archive.moveEntry(blogentry)
        
        ## get folder names out of entry date
        date = blogentry.Date()
        year = str(date.year())
        month = str(date.mm())
        day = str(date.dd())
        
        # entry should be out of drafts ...
        self.assertRaises(AttributeError, getattr, drafts, "blogentry")        
        
        # ... inside archive folders
        self.failUnless("blogentry" in blog.archive[year][month][day].objectIds())
    
        # and drafts has to be exist further
        self.failUnless("drafts" in blog.objectIds())

    def testMoveEntryWithinArchive(self):
        drafts = self.drafts
        archive = self.archive
        blog = self.blog
                
        ## create entry
        drafts.invokeFactory("EasyBlogEntry", id="blogentry", title="BlogEntry")
        
        ## commit to entry make movable
        transaction.commit(1)             

        blogentry = drafts.blogentry
        
        ## test moveEntry
        archive.moveEntry(blogentry)
        transaction.commit(1)             
        
        ## get folder names out of entry date
        date = blogentry.Date()
        year_1 = str(date.year())
        month_1 = str(date.mm())
        day_1 = str(date.dd())
 
        ## get moved entry
        blogentry = blog.archive[year_1][month_1][day_1]["blogentry"]
        
        ## set a new entry date
        new_date = date + 365
        
        blogentry.REQUEST.set("updateDate", "update_to_date")        
        blogentry.setEntryDate(new_date)
        
        ## move entry inside the archive
        archive.moveEntry(blogentry)

        ## get folder names out of entry date
        date = blogentry.Date()
        year = str(date.year())
        month = str(date.mm())
        day = str(date.dd())
        
        ## is it in new folders?
        self.failUnless("blogentry" in blog.archive[year][month][day].objectIds())
        
        ## are old folders delete?
        # self.assertRaises(AttributeError, getattr, blog.archive, year_1)
                                
    def testGetArchivesAsDicts(self):
        archive = self.archive
        
        archive.invokeFactory("EasyBlogArchiveFolder", id="2005", title="2005")
        year_folder = archive["2005"]

        # create some archives
        for month in range(0, 12):            
            year_folder.invokeFactory("EasyBlogArchiveFolder", id=month+1, title=month+1)
        
        
        expected_archives = [{'start': '2005/1/01', 'amount': 0, 'year': '2005', 'end': '2005/1/31', 'month': 'month_jan'},
                             {'start': '2005/2/01', 'amount': 0, 'year': '2005', 'end': '2005/2/28', 'month': 'month_feb'},
                             {'start': '2005/3/01', 'amount': 0, 'year': '2005', 'end': '2005/3/31', 'month': 'month_mar'},
                             {'start': '2005/4/01', 'amount': 0, 'year': '2005', 'end': '2005/4/30', 'month': 'month_apr'},
                             {'start': '2005/5/01', 'amount': 0, 'year': '2005', 'end': '2005/5/31', 'month': 'month_may'},
                             {'start': '2005/6/01', 'amount': 0, 'year': '2005', 'end': '2005/6/30', 'month': 'month_jun'},
                             {'start': '2005/7/01', 'amount': 0, 'year': '2005', 'end': '2005/7/31', 'month': 'month_jul'},
                             {'start': '2005/8/01', 'amount': 0, 'year': '2005', 'end': '2005/8/31', 'month': 'month_aug'},
                             {'start': '2005/9/01', 'amount': 0, 'year': '2005', 'end': '2005/9/30', 'month': 'month_sep'},
                             {'start': '2005/10/01', 'amount': 0, 'year': '2005', 'end': '2005/10/31', 'month': 'month_oct'},
                             {'start': '2005/11/01', 'amount': 0, 'year': '2005', 'end': '2005/11/30', 'month': 'month_nov'},
                             {'start': '2005/12/01', 'amount': 0, 'year': '2005', 'end': '2005/12/31', 'month': 'month_dec'}]

        
        is_archives = archive.getArchivesAsDicts()
        
        for i in range(0, len(expected_archives)):
            for key in expected_archives[i].keys():
                self.assertEqual(expected_archives[i][key], is_archives[i][key])
    
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestEasyBlogArchive))
    return suite

if __name__ == '__main__':
    framework()

