#
# Test EasyBlogComment
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.PloneTestCase import PloneTestCase
from Products.EasyBlog.tests import EasyBlogTestCase


from AccessControl import Unauthorized
from Products.CMFCore.WorkflowCore import WorkflowException

MSG = "<br><span>span</span><p>p</p><a href='http://www.test.de'>a</a><div>div</div>"
MSG_ALLOWED = "<br><span>span</span><p>p</p><a href='http://www.test.de'>a</a>&lt;div&gt;div&lt;/div&gt;"
MSG_DISALLOWED = "<br>&lt;span&gt;span&lt;/span&gt;&lt;p&gt;p&lt;/p&gt;&lt;a href='http://www.test.de'&gt;a&lt;/a&gt;<div>div</div>"                

class TestEasyBlogComment(EasyBlogTestCase.EasyBlogTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager', ])
        self.folder.invokeFactory("EasyBlog", id="blog", title="Blog", htmlTags=["<a>", "<p>", "<span>"], htmlAttributesHandling = "disallowed_attributes")                                  
        self.folder.blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry", title="Blogentry")
        self.folder.blog.drafts.blogentry.comments.invokeFactory('EasyBlogComment', id="blogcomment", title="Blogcomment", message=MSG)
    
        self.blog = self.folder.blog
        self.blogcomment = self.folder.blog.drafts.blogentry.comments.blogcomment
                        
    def testInvoking(self):
        c = self.blogcomment

        self.assertEqual(c.getId(), "blogcomment")
        self.assertEqual(c.title_or_id(), "Blogcomment")      
        self.assertEqual(c.getHtmlTags(), ("<a>", "<p>", "<span>"))
        self.assertEqual(c.getHtmlAttributesHandling(), "disallowed_attributes")
        
    def testGetMessage(self):                        
        comment = self.blogcomment
        
        # tags allowed
        self.folder.blog.setHtmlTagsHandling("allowed_tags")
        self.assertEqual(comment.getMessage(), MSG_ALLOWED)

        # tags disallowed
        self.folder.blog.setHtmlTagsHandling("disallowed_tags")
        self.assertEqual(comment.getMessage(), MSG_DISALLOWED)
        
    def testAlwaysReview(self):
        blog = self.folder.blog
        comment = self.blogcomment
        
        comment.setMessage("one, three, five ... ?")
        
        blog.setPatternsToReviewComments(["two", "four"])                        
        self.assertEqual(comment.alwaysReview(), False)
        
        # matching message
        blog.setPatternsToReviewComments(["one", "two"])                
        self.assertEqual(comment.alwaysReview(), True)          
    
        # matching name
        comment.message = ""    
        comment.title = "one"
        self.assertEqual(comment.alwaysReview(), True)          

        # matching email 
        comment.title = ""
        comment.email = "one@two.de"
        self.assertEqual(comment.alwaysReview(), True)          
        
        # matching homepage 
        comment.homepage = "www.one.de"
        self.assertEqual(comment.alwaysReview(), True)                          

    def testInterface(self):        
        from Products.EasyBlog.interfaces.IEasyBlogComment import IEasyBlogComment
        from Products.EasyBlog.content.EasyBlogComment import EasyBlogComment
        from Interface.Verify import verifyClass
        
        verifyClass(IEasyBlogComment, EasyBlogComment)
        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestEasyBlogComment))
    return suite

if __name__ == '__main__':
    framework()

