## Test EasyBlogEntry

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.EasyBlog.tests import EasyBlogTestCase
from DateTime import DateTime 

class TestEasyBlogEntry(EasyBlogTestCase.EasyBlogTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager', ])
        self.folder.invokeFactory("EasyBlog", id="blog", title="Blog")
        self.folder.blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry", title="Blogentry", text="Text", more="More", entryCategories=["general", ])        
        self.blogentry = self.folder.blog.drafts.blogentry
        
    def testTypeInformation(self):               
        portal_type = self.portal.portal_types        
        type_info = portal_type.getTypeInfo("EasyBlogEntry")
        
        self.assertEqual(type_info.global_allow, False)
        self.assertEqual(type_info.filter_content_types, True)
        self.assertEqual(type_info.allowed_content_types, ("EasyBlogTrackbacks", "EasyBlogComments"))        
        self.assertEqual(type_info.content_meta_type, "EasyBlogEntry")

        ## actions
        actions = [a.id for a in type_info._actions]        
        self.assertEqual(actions, ['view', 'edit', 'metadata', 'references', 'folderlisting'])
                
    def testInvoking(self):              
        blogentry = self.blogentry
        self.assertEqual(blogentry.id, "blogentry")
        self.assertEqual(blogentry.title, "Blogentry")
        self.assertEqual(blogentry.getText(), "<p>Text</p>")      ## have to get HTML back
        self.assertEqual(blogentry.getMore(), "<p>More</p>")      ## have to get HTML back
        self.assertEqual(blogentry.getUpdateDate(), "no_update")
        
        self.failUnless("trackbacks" in blogentry.objectIds())
        self.failUnless("comments" in blogentry.objectIds())

    ## Methods
    
    def testShowCommentsInView(self):
        """ """
        
        # default
        # blog is True / blogentry is default        
        self.assertEqual(self.blogentry.showCommentsInView(), True)
        
        self.folder.blog.showComments = False   
        self.assertEqual(self.blogentry.showCommentsInView(), False)
        
        # False
        self.blogentry.showComments = "no"
        self.assertEqual(self.blogentry.showCommentsInView(), False)
        
        # True
        self.blogentry.showComments = "yes"
        self.assertEqual(self.blogentry.showCommentsInView(), True)
        
    def testAllowCommentsInView(self):
        """ """
        # default
        # blog is True / blogentry is default
        self.assertEqual(self.blogentry.allowCommentsInView(), True)
        
        self.folder.blog.allowComments = False   
        self.assertEqual(self.blogentry.allowCommentsInView(), False)
        
        # No
        self.blogentry.allowComments = "no"
        self.assertEqual(self.blogentry.allowCommentsInView(), False)
        
        # Yes
        self.blogentry.allowComments = "yes"
        self.assertEqual(self.blogentry.allowCommentsInView(), True)

      
    def testShowTrackbacksInView(self):
        """ """
        # default
        # blog is True / blogentry is default
        self.assertEqual(self.blogentry.showTrackbacksInView(), True)
        
        self.folder.blog.showTrackbacks = False   
        self.assertEqual(self.blogentry.showTrackbacksInView(), False)
        
        # No
        self.blogentry.showTrackbacks = "no"
        self.assertEqual(self.blogentry.showTrackbacksInView(), False)
        
        # Yes
        self.blogentry.showTrackbacks = "yes"
        self.assertEqual(self.blogentry.showTrackbacksInView(), True)

      
    def testReceiveTrackbacksInView(self):
        """ """
        # default
        # blog is True / blogentry is default
        self.assertEqual(self.blogentry.receiveTrackbacksInView(), True)
        
        self.folder.blog.receiveTrackbacks = False   
        self.assertEqual(self.blogentry.receiveTrackbacksInView(), False)
        
        # No
        self.blogentry.receiveTrackbacks = "no"
        self.assertEqual(self.blogentry.receiveTrackbacksInView(), False)
        
        # Yes
        self.blogentry.receiveTrackbacks = "yes"
        self.assertEqual(self.blogentry.receiveTrackbacksInView(), True)


    def testUpdateDate(self): 
        """ """
        blogentry = self.blogentry        
        now = DateTime()
        
        # no_update
        blogentry.setEntryDate(now)     
        self.assertNotEqual(blogentry.getEntryDate(), now)
        
        # update_to_date
        blogentry.REQUEST.set("updateDate", "update_to_date")        
        date = DateTime("2005/07/23 11:11")        
        blogentry.setEntryDate(date)             
        self.assertEqual(blogentry.getEntryDate(), date)
        
        # update_to_now
        blogentry.REQUEST.set("updateDate", "update_to_now")        
        blogentry.setEntryDate(date)
        
        now = DateTime()
        now = "%s/%s/%s %s:%s" % (now.year(), now.month(), now.dd(), now.hour(), now.minute())
        
        ed = blogentry.getEntryDate()
        ed = "%s/%s/%s %s:%s" % (ed.year(), ed.month(), ed.dd(), ed.hour(), ed.minute())
    
        self.assertEqual(ed, now)

#     def test setEntryDate():
#         s. testUpdateDate
    
    def testDates(self):
        blogentry = self.blogentry
        self.assertEqual(blogentry.start(), blogentry.end())
        self.assertEqual(blogentry.start(), blogentry.Date())

        
    def testGetAmountOfComments(self):
        blogentry = self.blogentry
        self.assertEqual(blogentry.getAmountOfComments(), 0)

        for i in range(10):
            self.blogentry.comments.manage_addProduct["EasyBlog"].addEasyBlogComment(id="comment_%s" % i)
            c = self.blogentry.comments["comment_%s" % i]                        
            self.portal.portal_workflow.doActionFor(c, "publish")
            
        self.assertEqual(blogentry.getAmountOfComments(), 10)


    def testGetAmountOfTrackbacks(self):
        blogentry = self.blogentry
        self.assertEqual(blogentry.getAmountOfTrackbacks(), 0)
        
        for i in range(10):
            self.blogentry.trackbacks.manage_addProduct["EasyBlog"].addEasyBlogTrackback(id="trackback_%s" % i)
            t = self.blogentry.trackbacks["trackback_%s" % i]        
            self.portal.portal_workflow.doActionFor(t, "publish")
            
        self.assertEqual(blogentry.getAmountOfTrackbacks(), 10)

    def testGetEntryCategories(self):
        blogentry = self.blogentry
        categories  = blogentry.getEntryCategories()
        self.assertEqual(categories, ["general", ])
        
        # After the category is deleted
        blogentry.getBlog().categories.manage_delObjects(ids=["general", ])
        
        # the string is still in blogentry
        self.assertEqual(blogentry.entryCategories, (u"general", ))
        
        # but the method doesn't return the category anymore
        categories  = blogentry.getEntryCategories()        
        self.assertEqual(categories, [])
        
    ## Searching
    def testSearching(self):
        results = self.portal.portal_catalog(portal_type="EasyBlogEntry", SearchableText="Text")        
        self.assertEqual(results[0].getObject().getId(), "blogentry")

        results = self.portal.portal_catalog(portal_type="EasyBlogEntry", SearchableText="More")
        self.assertEqual(results[0].getObject().getId(), "blogentry")
    

    def testDeleteEntry(self):
        blog = self.folder.blog    
        blogentry = self.blogentry
        blogentry.delete()
        
        self.failIf("blogentry" in blog.objectIds())
        
        
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestEasyBlogEntry))
    return suite

if __name__ == '__main__':
    framework()

