#
# Test EasyBlogTrackback
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Testing import ZopeTestCase
from Products.EasyBlog.tests import EasyBlogTestCase
from Products.PloneTestCase import PloneTestCase

from AccessControl import Unauthorized
from Products.CMFCore.WorkflowCore import WorkflowException

MSG = "Das <p>ist</p> ein Test 1. Das ist ein Test 2"          

class TestEasyBlogTrackback(EasyBlogTestCase.EasyBlogTestCase):

    def afterSetUp(self):
        self.setRoles(['Manager', ])
        self.folder.invokeFactory("EasyBlog", id="blog", title="Blog", maxCharsInTrackbacks=0)
        self.folder.blog.drafts.invokeFactory('EasyBlogEntry', id="blogentry", title="Blogentry")
        self.folder.blog.drafts.blogentry.trackbacks.invokeFactory('EasyBlogTrackback', id="blogtrackback", title="Blogtrackback", url="http://www.test.de", excerpt=MSG)
        
        self.blogtrackback = self.folder.blog.drafts.blogentry.trackbacks.blogtrackback
                        
    def testInvoking(self):
        t = self.blogtrackback

        self.assertEqual(t.getId(), "blogtrackback")
        self.assertEqual(t.title_or_id(), "Blogtrackback")      
        self.assertEqual(t.getMaxCharsInTrackbacks(), 0)        
        self.assertEqual(t.getExcerpt(), MSG)        
        
    def testInterface(self):        
        from Products.EasyBlog.interfaces.IEasyBlogTrackback import IEasyBlogTrackback
        from Products.EasyBlog.content.EasyBlogTrackback import EasyBlogTrackback
        from Interface.Verify import verifyClass
        
        verifyClass(IEasyBlogTrackback, EasyBlogTrackback)

    def testGetExcerpt(self):
        t = self.blogtrackback
        self.assertEqual(t.getExcerpt(), MSG)
        
        self.folder.blog.setMaxCharsInTrackbacks(10)
        self.assertEqual(t.getExcerpt(), """Das ist ein <a title="Blogtrackback" href="http://www.test.de">More</a>""")

    def testAlwaysReview(self):
        blog = self.folder.blog
        trackback = self.blogtrackback
        
        trackback.setExcerpt("one, three, five ... ?")
        
        # not matching
        blog.setPatternsToReviewTrackbacks(["two", "four"])                        
        self.assertEqual(trackback.alwaysReview(), False)
        
        # matching excerpt
        blog.setPatternsToReviewTrackbacks(["one", "two"])                
        self.assertEqual(trackback.alwaysReview(), True) 
        
        # matching blogname
        trackback.excerpt = ""
        trackback.blogname = "one"
        self.assertEqual(trackback.alwaysReview(), True) 
        
        # matching url 
        trackback.blogname = ""
        trackback.url = "http://www.one.de"
        self.assertEqual(trackback.alwaysReview(), True)         
                         
def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestEasyBlogTrackback))
    return suite

if __name__ == '__main__':
    framework()

