from HTMLParser import HTMLParser

class CommentHTMLParser(HTMLParser):
    """Let only allowed tags thru; 
       all others will be displayed as plain text"""
    
    def __init__(self, tags, tags_handling, attributes, attributes_handling):
        HTMLParser.__init__(self)
        
        self._tags_handling = tags_handling        
        self._tags = self.prepare_tags(tags)        
        self._attributes = attributes
        self._attributes_handling = attributes_handling

        self._html      = ""
        self._tagbegin = ""
        self._tagend   = ""
        self._allowed_tag = False
                
    def prepare_tags(self, tags):
        """Transforms the allowed tags to lower case and remove <, >"""
        
        tags = [tag.replace('<', '') for tag in tags ]
        tags = [tag.replace('>', '') for tag in tags ]
        tags = [tag.lower() for tag in tags]
    
        return tags
    
    def prepare_tag(self, tag):
        """Prepares tag in dependence is allowed or not"""
                
        if tag == "br":
            self._allowed_tag = True
        elif self._tags_handling == "allowed_tags":
            if tag in self._tags:
                self._allowed_tag = True
            else:
                self._allowed_tag = False
        else: 
            if tag in self._tags:
                self._allowed_tag = False
            else:
                self._allowed_tag = True                
                
        if self._allowed_tag == True:
            self._tagbegin = "<"
            self._tagend   = ">"
        else:
            self._tagbegin = "&lt;"
            self._tagend   = "&gt;"
            
    def getHTML(self):
        """Returns parsed HTML"""
        return self._html
            
    def handle_data(self, data):
        """Overwritten"""
        self._html += data
        
    def handle_starttag(self, tag, attrs):
        """Overwritten"""        
        self.prepare_tag(tag)        
        self._html += self._tagbegin + tag     
        
        # Attributes
        for x in range(len(attrs)):
            if self._allowed_tag:
                if self._attributes_handling == "allowed_attributes" and attrs[x][0] not in self._attributes:
                    continue
                elif self._attributes_handling == "disallowed_attributes" and attrs[x][0] in self._attributes:
                    continue
            
            self._html += (" %s='%s'" % (attrs[x][0], attrs[x][1]))            
        
        self._html += self._tagend
            
    def handle_endtag(self, tag):
        """Overwritten"""        
        self.prepare_tag(tag)
        if tag != "br":
            self._html += self._tagbegin + '/' + tag + self._tagend    

