import httplib, urllib, urlparse
from httplib import HTTPException, BadStatusLine

from DateTime import DateTime
from random import random
from Products.EasyBlog.utils.UrlHTMLParser import UrlHTMLParser
from Products.EasyBlog.utils.RdfHTMLParser import RdfHTMLParser
from Products.EasyBlog.content.EasyBlogTrackback import EasyBlogTrackback
from HTMLParser import HTMLParseError
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowTool import WorkflowException

class TrackbackManager:
    """Responsible class for all trackback tasks
    """

    # Publics
    def addTrackback(self, entry):
        """Adds a incoming trackback"""
        
        if hasattr(entry, "trackbacks") == False:
            entry.manage_addProduct["EasyBlog"].addEasyBlogTrackbacks(id="trackbacks", title="Trackbacks")

        REQUEST = entry.REQUEST
        RESPONSE = REQUEST.RESPONSE
        RESPONSE.setStatus("200")    

        # get url here to check whether entry already exist
        url = REQUEST.get('url')        

        if self._existTrackback(entry, url) == True:
            RESPONSE.write(self._getResponseMessage(1, "Entry already exists"))            
        else:
            id = self._generateUniqueId() 
            title = REQUEST.get("title")
            excerpt = REQUEST.get('excerpt', '')
            blogname = REQUEST.get("blog_name", '')

            # XXX: check it: doesn't work with anonymous user
            #entry.trackbacks.manage_addProduct["EasyBlog"].addEasyBlogTrackback(id=id, title=title, url=url, excerpt=excerpt, blogname=blogname)
        
            t = EasyBlogTrackback(id)
            entry.trackbacks._setObject(id, t) 

            trackback = entry.trackbacks[id]
            
            trackback.setTitle(title)
            trackback.setUrl(url)
            trackback.setExcerpt(excerpt)
            trackback.setBlogname(blogname)
            
            portal_workflow = getToolByName(trackback, "portal_workflow")                
            try:
                if entry.getBlog().getModerateTrackbacks() == True:
                    portal_workflow.doActionFor(trackback, "submit")
                else:
                    # check content of trackback
                    if trackback.alwaysReview() == True: 
                        portal_workflow.doActionFor(trackback, "submit")
                    else:
                        portal_workflow.doActionFor(trackback, "publish")
                        
            except WorkflowException, e:
                entry.trackbacks._delObject(id)                    
                RESPONSE.write(self._getResponseMessage(1, "Internal Error: Couldn't create entry"))
            else:
                # sendMail
                ebt = getToolByName(entry, "easyblog_tool")
                ebt.sendMail(entry.getBlog(), "New Trackback", title, blogname, "", url, trackback.getIpaddress(), excerpt)
                
                RESPONSE.write( self._getResponseMessage(0) )

    def sendTrackbacks(self, entry):
        """Send trackbacks"""
        
        self.log = entry.getTrackbackLog().getNewLog()        
        self.log.addTitle("%s" % DateTime())
                
        url = entry.absolute_url()
        title = entry.Title()        
        excerpt = entry.getText()
        more = entry.getMore()
        blog_name = entry.getBlog().Title()
        target_urls = entry.getTrackbackURLs()               # manually added urls
        auto_trackback = entry.getAutodetectTrackbacks()
                
        body  = urllib.urlencode({"title":title, "url":url, "excerpt":excerpt, "blog_name":blog_name})
        headers = {"Content-type": "application/x-www-form-urlencoded" }
        
        if auto_trackback == False:
            auto_urls = []
            self.log.addLine2("Autodetect URLs", "Turned off")
        else:
            auto_urls = self._autodetectUrls(excerpt + " " + more)
            
            # just log it
            # XXX: profile it
            self.log.addTitle("Autodetected URLs")            
            for url in auto_urls:
                self.log.addLine(url)
        
        # all entered urls only once        
        unique_urls = self._getUniqueUrls(auto_urls, target_urls)        
        
        # and again
        # XXX: profile it        
        self.log.addTitle("All URLs")                        
        for url in unique_urls.keys():        
            self.log.addLine(url)
        
        # get the trackback urls out of the entered urls (trackback:ping) 
        self.log.addTitle("Parse For Trackback-Pings")
        tb_urls = self._getTrackbackUrls(unique_urls)
                
        # finally send Trackbacks        
        self._sendTrackbacks(body, headers, tb_urls)
        
            
    ##########################################################################################################################################
    ### Privates 
    
    def _getResponseMessage(self, nr, message=""):
        """Generates a proper reponse message"""
        if nr == 0:
            return '<?xml version="1.0" encoding="utf-8"?><response><error>%s</error></response>' % nr
        else:
            return '<?xml version="1.0" encoding="utf-8"?><response><error>%s</error><message>%s</message></response>' % (nr, message)
    
    def _generateUniqueId(self, type_name=None):
        """Generates a unique id"""
        
        # copied from CMFPlone
        now=DateTime()
        time='%s.%s' % (now.strftime('%Y-%m-%d'), str(now.millis())[7:])
        rand=str(random())[2:6]
        prefix=''
        suffix=''

        if type_name is not None:
            prefix = type_name.replace(' ', '_')+'.'
            prefix=prefix.lower()

        return prefix+time+rand+suffix        
    
    def _existTrackback(self, entry, url):
        """Checks if an entry already exists"""
        tb_exists = False
        for tb in entry.trackbacks.objectValues():
            if tb.getUrl() == url:
                tb_exists = True
                
        return tb_exists 
    
    def _autodetectUrls(self, html):
        """Autodetects urls out of a given chunk of html"""
        url_parser = UrlHTMLParser()
        
        try:
            url_parser.feed(html)
        except HTMLParseError:            
            self.log.addLine2("Autodetect URLs", "Parse Error")
        else:
            self.log.addLine2("Autodetect URLs", "Success")
        
        self.log.addLine2("", "Found: %s" % len(url_parser.urls))
        
        return url_parser.urls

    def _getUniqueUrls(self, target_urls, auto_urls):
        """Remove double URLs"""

        # remove doubles
        unique_urls = dict()
        for url in target_urls:
            unique_urls[url] = 1

        for url in auto_urls:
            unique_urls[url] = 1

        return unique_urls
    
    def _getTrackbackUrls(self, urls):
        """Try to get the trackback urls out of the given urls and the received rdf-tag

           The passed URLs are URLs which where found in the trackbackurl or text/more-fields  
           To get the trackback urls, one have to request the urls and parse the rdf tag for a 
           trackback:ping - attribute
        """
        tb_urls = dict()
        for url in urls.keys():
            tb_url = self._getTrackbackUrl(url)
            
            if tb_url is not None:
                tb_urls[tb_url] = 1
                
        return tb_urls

    def _getTrackbackUrl(self, url):
        """Parses the response of a given url (permalink of an receiving entry) and return the trackback_url
           which was inside the trackback:ping-attribute of the response of the request to the permalink"""
        
        # for the case, the url is not in the format http://www. ...
        # XXX: Okay for internal URL because permalink and trackback url are equal
        try:
            host = urlparse.urlparse(url)[1]                    
        except IndexError:
            self.log.addLine2(url, "Invalid URL. Skipped.")
            return None
                    
        path = url.replace("http://%s" % host, "")

        try:
            conn = httplib.HTTPConnection(host)
            conn.request("GET", path)
        except HTTPException:
            self.log.addLine2(url, "HTTP Error. Skipped.")
            return None
        except IndexError:
            self.log.addLine2(url, "Invalid URL. Skipped.")
            return None
        except:
            self.log.addLine2(url, "No Connection. Skipped.")
            return None
            
        # get response
        try:
            response = conn.getresponse()                               
        except BadStatusLine:                                
            self.log.addLine2(url, "Bad Status Line. Skipped.")            
            return None
            
        html = response.read()
        
        self.log.addLine2(url, str(response.status))
                
        rdf_parser = RdfHTMLParser(url)
                
        # for the case, that the received page is not valid
        try: 
            rdf_parser.feed(html) 
        except HTMLParseError:
            self.log.addLine2("", "Parse Error. Take: %s" % url)
            return url
            
        # if a trackback:ping is found return it, 
        # else return the origin URL (for the case the receiving page hasn't implement a RDF-tag)
        if rdf_parser.tb_url is not None:
            self.log.addLine2("", "Found: %s" % rdf_parser.tb_url)
            return rdf_parser.tb_url
        else:
            self.log.addLine2("", "Nothing Found. Take: %s" % url)            
            return url

    def _sendTrackbacks(self, body, headers, urls):
        """Send Trackbacks to trackback:ping url"""
        
        self.log.addTitle("Send Trackbacks")

        if len(urls.keys()) == 0:
            self.log.addLine("Nothing to send.")            
            
        for url in urls.keys():  
            host = urlparse.urlparse(url)[1]                    
            path = url.replace("http://%s" % host, "")            
            try: 
                conn = httplib.HTTPConnection(host)
                conn.request("POST", path, body, headers)
            except HTTPException:
                self.log.addLine2(url, "HTTP Error")
            else:
                response = conn.getresponse()               
                self.log.addLine2(url, str(response.status))
